package require tcltest

tcltest::configure -verbose [list body pass error]

tcltest::test nmc-load-library {Make sure library can be found/loaded} \
	-body {
		package require nmc
	} \
	-result 0.2

namespace import nmc::nmc

tcltest::test nmc-max-depth-equality {simple string equality at maximum n-depth} \
	-body {
		nmc 0 asdf asdf
	} \
	-result 0.000000000000000000

tcltest::test nmc-small-commutative {s1 and s2 positions do not matter} \
	-body {
		expr {[nmc 0 aaaaa zzzzz] == [nmc 0 zzzzz aaaaa]}
	} \
	-result 1

tcltest::test nmc-large-equality {large string equality} \
	-setup {
		set random [open /dev/urandom r]
		set r_size [expr 0x10000]
		set s1 [read $random $r_size]
		set s2 [read $random $r_size]
		close $random
	} \
	-body {
		nmc 0 $s1 $s1
	} \
	-result 0.000000000000000000


tcltest::test nmc-large-commutative {s1 and s2 postions do not matter for large strings} \
	-body {
		expr {[nmc 0 $s1 $s2] == [nmc 0 $s2 $s1]}
	} \
	-result 1


tcltest::test nmc-min-depth-equality {s1 == s1 when n=1 } \
	-body {
		nmc 1 aaaaa aaaaa
	} \
	-result 0.000000000000000000

tcltest::test nmc-min-depth-equality-large {s1 == s1 when n=1 for large strings} \
	-body {
		nmc 1 $s1 $s1
	} \
	-result 0.000000000000000000

tcltest::test nmc-1-percent-depth-equality-large {s1 == s1 when n=1 for large strings} \
	-body {
		nmc 1% $s1 $s1
	} \
	-result 0.000000000000000000

